// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#ifndef __CONSTRAINTS_H
#define __CONSTRAINTS_H 1

// this module contains constraints on a surface F

#include <iostream>
#include <geotyp.h>

#include "param.h"
#include "bspline.h"

//-----------------------------------------------------------------------//
//                          PointToPointConstraint                       //
//-----------------------------------------------------------------------//

class PointToPointConstraint {
  public:
    double3 point;
    double2 u;
  public:
    PointToPointConstraint(const double2& u0, const double3& p): point(p), u(u0) {}
};

//-----------------------------------------------------------------------//
//                          PointToLineConstraint                        //
//-----------------------------------------------------------------------//

class PointToLineConstraint {
  public:
    Line3D line;
    double2 u;
  public:
    PointToLineConstraint(double2 u0, const Line3D& l): line(l), u(u0) {}
};

//-----------------------------------------------------------------------//
//                          PointToPlaneConstraint                       //
//-----------------------------------------------------------------------//

class PointToPlaneConstraint {
  public:
    Plane3D plane;
    double2 u;
  public:
    PointToPlaneConstraint(const double2& u0, const Plane3D& p): plane(p), u(u0) {}
};

//-----------------------------------------------------------------------//
//                          NormalToVectorConstraint                     //
//-----------------------------------------------------------------------//

class NormalToVectorConstraint {
  public:
    double3 normal;
    double2 u;
  public:
    NormalToVectorConstraint(const double2& u0, const double3& n): normal(n), u(u0) {}
};

//-----------------------------------------------------------------------//
//                          HorizontalCurveConstraint                    //
//-----------------------------------------------------------------------//

class HorizontalCurveConstraint {
  public:
    int2 u;
    double v0;
    BSpline curve;
  public:
    HorizontalCurveConstraint(const int2& u0, double v00, char* FileName):
      u(u0), v0(v00), curve(FileName) {}
};

//-----------------------------------------------------------------------//
//                          VerticalCurveConstraint                      //
//-----------------------------------------------------------------------//

class VerticalCurveConstraint {
  public:
    double u0;
    int2 v;
    BSpline curve;
  public:
    VerticalCurveConstraint(double u00, const int2& v0, char* FileName):
      u0(u00), v(v0), curve(FileName) {}
};

//-----------------------------------------------------------------------//
//                          BoundaryConstraint                           //
//-----------------------------------------------------------------------//

class BoundaryConstraint {
  public:
    BoundaryConstraint() {}
};

#endif
