// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// Sat 06-22-1996

#ifndef __CONTROLPOINTSURFACE_H
#define __CONTROLPOINTSURFACE_H 1

#include "parsurf.h"
#include <dvector.h>
#include <polymesh.h>
#include <bbox.h>

//-----------------------------------------------------------------------//
//                          ControlPointSurface                          //
//-----------------------------------------------------------------------//
class ControlPointSurface: public ParametricSurface
// this class represents a surface F: omega in R^2 -> R^3 that can be
// represented as follows:
//                      n-1
//             F(u,v) = sum   N (u,v)  P
//                      i=0    i        i
//
// where {P_i} is a set of 3D control points
{
  public:
    virtual int NumControlPoints() const = 0;
    // returns the number of control points of the surface
    
    virtual const double3& GetControlPoint(int i) const = 0;
    // returns the i-th control point of the surface (0 <= i < NumControlPoints())

    virtual void SetControlPoint(int i, const double3& point) = 0;
    // sets the value of the i-th control point (0 <= i < NumControlPoints())

    virtual void GetControlPointMesh(PolygonMesh& mesh) const;
    // generates a mesh of the control polyhedron

    BoundingBox3D GetControlPointBBox() const;
    // returns the bounding box of the control points

    DoubleVector GetControlPointVector() const;
    // computes the concatenation vector of the control points

    void SetControlPointVector(DoubleVector& x);
    // initializes the control points with the concatenation vector x

    void Perturb(double d);
    // adds a random number in the interval [-d,d] to all components
    // of all control points of the surface
};

#endif

