// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// 1-2-95

#ifndef __MYMANIP_H
#define __MYMANIP_H

#include <iostream>
#include <iomanip>
#include <ctype.h>
#include <string.h>

//-----------------------------------------------------------------------//
//                          skipeol                                      //
//-----------------------------------------------------------------------//
// this manipulator skips characters until the end of a line
// N.B. the end of line character '\n' is extracted from the stream
inline std::istream& skipeol(std::istream& is)
{
  while (!is.eof()) {
    char c;
    is.get(c);
    if (isspace(c))
      if (c == '\n') return is;
  }
  return is;
}

//-----------------------------------------------------------------------//
//                          skipbracket                                  //
//-----------------------------------------------------------------------//
// this manipulator skips whitespace characters until the end of a line
// if the next character is one of the following [](){} it is also extracted
// from the stream
inline std::istream& skipbracket(std::istream& is)
{
  while (!is.eof()) {
    char c;
    is.get(c);
    if (isspace(c)) {
      if (c == '\n') {
        is.putback(c);
        break;
      }
    }
    else {
      if (!strchr("[](){}", c))
        is.putback(c);
      break;
    }
  }
  return is;
}

//-----------------------------------------------------------------------//
//                          skipcomma                                    //
//-----------------------------------------------------------------------//
// this manipulator skips whitespace characters until the end of a line
// if the next character is a comma ',' it is also extracted from the stream
inline std::istream& skipcomma(std::istream& is)
{
  while (!is.eof()) {
    char c;
    is.get(c);
    if (isspace(c)) {
      if (c == '\n') {
        is.putback(c);
        break;
      }
    }
    else {
      if (c != ',')
        is.putback(c);
      break;
    }
  }
  return is;
}

#endif
