// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// Sat 06-15-1996

#ifndef __POLYGONMESH_H
#define __POLYGONMESH_H 1

#include <iostream>
#include <listtmpl.h>
#include <int3.h>
#include <int4.h>
#include <double3.h>

//-----------------------------------------------------------------------//
//                          PolygonMesh
//-----------------------------------------------------------------------//

class PolygonMesh {
// This class represents a set of 3D triangles and quadrangles;
  protected:
    List<double3> points;
    List<int3> triangles;
    List<int4> quadrangles;

  public:
    PolygonMesh() {}
    // default constructor

    PolygonMesh(const PolygonMesh&) {}
    // copy constructor

    ~PolygonMesh() {}
    // destructor

    int NumPoints() const { return points.Count; }
    int NumTriangles() const { return triangles.Count; }
    int NumQuadrangles() const { return quadrangles.Count; }

    const double3& Point(int i) { return points[i]; }
    const int3& Triangle(int i) { return triangles[i]; }
    const int4& Quadrangle(int i) { return quadrangles[i]; }

    void Insert(const double3& point) { points.Insert(new double3(point)); }
    void Insert(const int3& triangle) { triangles.Insert(new int3(triangle)); }
    void Insert(const int4& quadrangle) { quadrangles.Insert(new int4(quadrangle)); }

    void SaveVRML(std::ostream& to, const double3& pointcolor, const double3& linecolor,
                  double pointradius, double lineradius) const;
    // saves the mesh as a serie of cylinders and spheres in the specified colors

    void SaveVRML(char* FileName, const double3& pointcolor, const double3& linecolor,
                  double pointradius, double lineradius) const;
    // saves the mesh as a serie of cylinders and spheres in the specified colors

    void Show() const;
    // writes the contents to standard out

    friend std::ostream& operator<<(std::ostream&, const PolygonMesh&);
    friend std::istream& operator>>(std::istream&, PolygonMesh&);
};

#endif
