// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// Tue 06-18-1996

#ifndef __SURFMESH_H
#define __SURFMESH_H 1

#include <iostream>
#include <sample.h>
#include <listtmpl.h>
#include <colorfnc.h>
#include <int3.h>
#include <int4.h>
#include <dvector.h>

//-----------------------------------------------------------------------//
//                          SurfaceMesh
//-----------------------------------------------------------------------//

class SurfaceMesh {
// This class represents a set of 3D triangles and quadrangles; in each
// vertex a SurfSample is stored. This means that normal directions and
// principal curvatures are available.
  protected:
    List<SurfSample> samples;
    List<int3> triangles;
    List<int4> quadrangles;

  public:
    SurfaceMesh() {}
    // default constructor

    SurfaceMesh(const SurfaceMesh&) {}
    // copy constructor

    ~SurfaceMesh() {}
    // destructor

    int NumSamples() const { return samples.Count; }
    int NumTriangles() const { return triangles.Count; }
    int NumQuadrangles() const { return quadrangles.Count; }

    DoubleVector GetTotalCurvatureValues() const;
    // returns a vector with the values of the total curvature of the vertices
    
    DoubleVector GetSimpleTotalCurvatureValues() const;
    // returns a vector with the values of the simple total curvature of the vertices

    const SurfSample& Sample(int i) const { return samples[i]; }
    const int3& Triangle(int i)  const{ return triangles[i]; }
    const int4& Quadrangle(int i)  const{ return quadrangles[i]; }

    void Insert(const SurfSample& sample) { samples.Insert(new SurfSample(sample)); }
    void Insert(const int3& triangle) { triangles.Insert(new int3(triangle)); }
    void Insert(const int4& quadrangle) { quadrangles.Insert(new int4(quadrangle)); }

    void SaveVRML(std::ostream& to, const Function3D& colorfunc) const;
    void SaveVRML(std::ostream& to, const double3& rgb) const;
    void SaveVRML(std::ostream& to, const double3& rgb0, const double3& rgb1) const;
    void SaveVRML(char* FileName, const Function3D& colorfunc) const;
    void SaveVRML(char* FileName, const double3& rgb) const;
    void SaveVRML(char* FileName, const double3& rgb0, const double3& rgb1) const;
    void SaveOffsetVRML(std::ostream& to, const Function3D& colorfunc, double d) const;
    void SaveOffsetVRML(std::ostream& to, const double3& rgb, double d) const;

    void Show() const;
    // writes the contents to standard out

    friend std::ostream& operator<<(std::ostream&, const SurfaceMesh&);
    friend std::istream& operator>>(std::istream&, SurfaceMesh&);
};

#endif
