// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include <poly.h>
#include <bipoly.h>
#include <fstream>
#include <pause.h>
#include <geotyp.h>

void TestPoly()
{
  std::ifstream from("poly.dat");

  Polynomial p;
  from >> p;

  Polynomial dp = p.Derivative(1);
  Polynomial ddp = p.Derivative(2);
  Polynomial P = p.Primitive();

  std::cout << "p = "; p.Show();
  std::cout << "dp = "; dp.Show();
  std::cout << "ddp = "; ddp.Show();
  std::cout << "P = "; P.Show();

  std::cout << "P' - p = "; (P.Derivative() - p).Show();
}

void TestBiPoly()
{
  std::ifstream from("testpoly.dat");

  BiPolynomial p;
  from >> p;
  std::cout << "p = "; p.Show();

  std::cout << "p*p = "; (p*p).Show();

  Polynomial p0;
  from >> p0;
  std::cout << "p0 = "; p0.Show();
  
  Polynomial p1;
  from >> p1;
  std::cout << "p1 = "; p1.Show();
  
  BiPolynomial p0p1(p0,p1);
  std::cout << "p0p1 = "; p0p1.Show();

  BiPolynomial q = p;
  std::cout << "q = "; q.Show();

  q += p;
  std::cout << "q = "; q.Show();

  BiPolynomial dp = p.Derivative(1,0);
  std::cout << "dp = "; dp.Show();

  q += p;
  std::cout << "q = "; q.Show();

  BiPolynomial ddp = p.Derivative(2,0);
  std::cout << "ddp = "; ddp.Show();

  BiPolynomial P = p.Primitive(1,0);
  std::cout << "P = "; P.Show();

  BiPolynomial Pxy = p.Primitive(1,1);
  std::cout << "Pxy = "; Pxy.Show();

  BiPolynomial pp  = Pxy.Derivative(1,1);
  std::cout << "pp = "; pp.Show();

  BiPolynomial f = p;

  f += dp;
  f -= dp;

  q = P.Derivative(1,0);
  std::cout << "q ="; q.Show();

  BiPolynomial qq = P.Derivative(1,0) - p;
  std::cout << "qq =" << std::endl; qq.Show();

  std::cout << "dP/dx - p = ";
  (P.Derivative(1,0) - p).Show();

  std::cout << "pp - p = "; (pp-p).Show();
}

void TestBiPoly2()
{
  std::ifstream from("bipoly.dat");

  BiPolynomial p;
  from >> p;
  std::cout << "p = "; p.Show();
  
  double x0 = 0;
  double x1 = 1;
  double a0 = 0;
  double a1 = 0;
  double b0 = 1;
  double b1 = 0;

  VerticalTrapezoid2D gamma(x0,x1,a0,a1,b0,b1);
  for (int i=0; i<4; i++)
    std::cout << i << " " << gamma.Point(i) << std::endl;
    
  std::cout << "center: " << gamma.Center() << std::endl;
  
  double s = p.Integrate(gamma);
  std::cout << "s = " << s << std::endl;
}

int main()
{
//  TestPoly();
  TestBiPoly();

  return 0;
}
