// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// 8-5-95
// nog:
// -> test of df(x) = 0

#include <newton.h>
#include <cmath>

// result:
//
// if a solution is found within maxiter iterations:
//    returnvalue:    true
//    solution:       the solution
//
// if no solution is found within maxiter iterations: 
//    returnvalue:    false
//                    let x[k] be the final value of the iteration process;
//    solution:       a    if x[k] < a 
//                    x[k] if a <= x[k] <= b
//                    b    if x[k] > c 


int FindZero(Polynomial& f, double x0, double a, double b, double tol, int maxiter, double& solution)
{
  double x, dx, fx, dfx;

  Polynomial df = f.Derivative();

//  std::cout << "FindZero():" << std::endl;
//  std::cout << "f = " << f << std::endl;
//  std::cout << "df = " << df << std::endl;

  x = x0;

//  std::cout << "x = " << x << " f(x) = " << f(x) << " df(x) = " << df(x) << std::endl;

  for (int i=0; i<maxiter; i++) {
    fx = f(x);
    dfx = df(x);
    dx = - fx / dfx;

    x += dx;
//    std::cout << "x = " << x << " f(x) = " << fx << " df(x) = " << dfx << std::endl;

    if ((x<a) || (x>b) || (fabs(dx/x) < tol)) break;
  }

  if ((a<=x) && (x<=b) && (fabs(dx/x) < tol)) {
    solution = x;
    return 1;
  }
  else {
    if (x<a) x=a;
    if (x>b) x=b;
    solution = x;
    return 0;
  }
}

int Newton(const Function1D& f, double x0, double a, double b, double tol, int maxiter, double& solution)
{
  double x, dx, fx, dfx;

  x = x0;

std::cout << "NEWTON" << std::endl;
std::cout << "a,b = " << a << " " << b << std::endl;
std::cout << "tol = " << tol << std::endl;

  for (int i=0; i<maxiter; i++) {
std::cout << "i = " << i << std::endl;
std::cout << "x = " << x << std::endl;
    fx = f(x);
std::cout << "f(x) = " << fx << std::endl;
    dfx = f.Deriv(x);
std::cout << "f'(x) = " << dfx << std::endl;
    dx = - fx / dfx;
std::cout << "dx = " << dx << std::endl;

    x += dx;

    if ((x<a) || (x>b) || (fabs(dx/x) < tol)) break;
  }

std::cout << "finished loop" << std::endl;
std::cout << "f(x) = " << f(x) << std::endl;

  if ((a<=x) && (x<=b) && (fabs(dx/x) < tol)) {
    solution = x;
    return 1;
  }
  else {
    if (x<a) x=a;
    if (x>b) x=b;
    solution = x;
    return 0;
  }
}

