// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

#include "nieuw.h"
#include <cmath>

double3x3 GetUnitMatrix3D()
// This function returns the 3 dimensional unit matrix
{
  return double3x3( 1, 0, 0,
                    0, 1, 0,
                    0, 0, 1  );
}

double3x3 GetCrossProductMatrix(const double3& n)
// This function returns a matrix A such that Ax = n^x for an
// arbitrary vector x.
{
  return double3x3(  0  , -n[2],  n[1],
                    n[2],   0  , -n[0],
                   -n[1],  n[0],   0    );
}

double3x3 GetAxialRotationMatrix(const double3& n, double phi)
// This function returns the orthogonal 3x3 matrix A that
// represents the rotation around the axis through n over
// the angle phi.
{
  double3x3 Result;

  double cosphi = cos(phi);
  double sinphi = sin(phi);
  double3 nunit = n.Unit();

  return   cosphi   * GetUnitMatrix3D() +
         (1-cosphi) * DyadeMatrix(nunit,nunit) +
           sinphi   * GetCrossProductMatrix(nunit);
}

double3x3 GetVectorRotationMatrix(const double3& a, const double3& b)
// This function returns an orthogonal 3x3 matrix A such that
// A (a.Unit()) = b.Unit() for two nonzero vectors a and b
{
  double denom = a.Length()*b.Length();
  double cosphi = (a,b) / denom;
  double sinphi = (a^b).Length() / denom;
  double3 nunit = (a^b).Unit();

  return   cosphi   * GetUnitMatrix3D() +
         (1-cosphi) * DyadeMatrix(nunit,nunit) +
           sinphi   * GetCrossProductMatrix(nunit);
}

