// Copyright (c) 1994-2010: Wieger Wesselink <j.w.wesselink@tue.nl>
//
// Distributed under the Boost Software License, Version 1.0.
// (See accompanying file LICENSE_1_0.txt or copy at
// http://www.boost.org/LICENSE_1_0.txt)

// Sat 06-15-1996

#include "polymesh.h"
#include <fstream>
#include <vrml.h>

//-----------------------------------------------------------------------//
//                          PolygonMesh                                  //
//-----------------------------------------------------------------------//

void PolygonMesh::SaveVRML(std::ostream& to, const double3& pointcolor, const double3& linecolor,
                           double pointradius, double lineradius) const
// saves the mesh as a serie of cylinders and spheres in the specified colors
{
  VRMLWriteBeginSeparator(to);
  VRMLWriteColor(to,linecolor);

  int i;
  
  for (i=0; i<triangles.Count; i++) { 
    VRMLWriteCylinder(to, points[triangles[i][0]], points[triangles[i][1]], lineradius);
    VRMLWriteCylinder(to, points[triangles[i][1]], points[triangles[i][2]], lineradius);
    VRMLWriteCylinder(to, points[triangles[i][2]], points[triangles[i][0]], lineradius);
  }

  for (i=0; i<quadrangles.Count; i++) { 
    VRMLWriteCylinder(to, points[quadrangles[i][0]], points[quadrangles[i][1]], lineradius);
    VRMLWriteCylinder(to, points[quadrangles[i][1]], points[quadrangles[i][2]], lineradius);
    VRMLWriteCylinder(to, points[quadrangles[i][2]], points[quadrangles[i][3]], lineradius);
    VRMLWriteCylinder(to, points[quadrangles[i][3]], points[quadrangles[i][0]], lineradius);
  }

  for (i=0; i<points.Count; i++)
    VRMLWriteSphere(to, points[i], pointradius, pointcolor);

  VRMLWriteEndSeparator(to);
}

void PolygonMesh::SaveVRML(char* FileName, const double3& pointcolor, const double3& linecolor,
                           double pointradius, double lineradius) const
// saves the mesh as a serie of cylinders and spheres in the specified colors
{
  std::ofstream to(FileName);
  if (!to)
    std::cerr << "Could not open file " << FileName << std::endl;
  else {
    VRMLWriteFileHeader(to); 
    SaveVRML(to, pointcolor, linecolor, pointradius, lineradius);
    to << "}" << std::endl << std::endl;
  }  
}

void PolygonMesh::Show() const
// writes the contents to standard out
{
  std::cout << "PolygonMesh::Show()" << std::endl;
  std::cout << " numpoints:      " << points.Count << std::endl;
  std::cout << " numtriangles:   " << triangles.Count << std::endl;
  std::cout << " numquadrangles: " << quadrangles.Count << std::endl;
  
  for (int i=0; i<triangles.Count; i++)
    std::cout << "triangle " << triangles[i] << std::endl;
    
  for (int i=0; i<quadrangles.Count; i++)
    std::cout << "quadrangle " << quadrangles[i] << std::endl;
}

std::ostream& operator<<(std::ostream& to, const PolygonMesh&) 
{
  return to << "this is a PolygonMesh" << std::endl;
}

std::istream& operator>>(std::istream& from, PolygonMesh&) 
{
  return from;
}

